package com.persagy.ems.pojo.wz;

import java.util.Date;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.persagy.core.annotation.Column;
import com.persagy.core.annotation.Dimension;
import com.persagy.core.annotation.Entity;
import com.persagy.core.annotation.Index;
import com.persagy.core.annotation.Table;
import com.persagy.core.enumeration.Schema;
import com.persagy.core.enumeration.EMSDimension;
import com.persagy.core.mvc.pojo.BusinessObject;

/**
 *	灾害预警
 */
@Dimension(dimension=EMSDimension.Building)
@Entity(name = "DisasterWarning")
@Table(name = "t_disaster", comment = "灾害预警信息", schema = Schema.EMS, indexes = {
		@Index(columns={"c_city_Id", "c_pub_time", "c_type", "c_level"}), 
		@Index(columns={"c_name", "c_pub_time", "c_type", "c_level"})})
public class DisasterWarning extends BusinessObject{
	private static final long serialVersionUID = -3666854586091234686L;

	@JsonProperty("cityId")
	@Column(order = 0, name = "c_city_Id", length = 40, nullable = false, comment = "城市ID")
	private String cityId;

	@Column(order = 1, name = "c_name", length = 40, nullable = false, comment = "城市名称")
	@JsonProperty("name")
	private String name;

	@Column(order = 2, name = "c_path", length = 50, nullable = false, comment = "隶属层级，从小到大")
	@JsonProperty("path")
	private String path;
	
	@Column(order = 3, name = "c_pub_time", length = 0, nullable = false, comment = "官网发布时间")
	@JsonProperty("pubTime")
	private Date pubTime;
	
	@Column(order = 4, name = "c_insert_time", length = 0, nullable = false, comment = "数据插入时间")
	@JsonProperty("insertTime")
	private Date insertTime;

	@Column(order = 30, name = "c_title", length = 100, nullable = false, comment = "预警标题")
	@JsonProperty("title")
	private String title;
	
	@Column(order = 40, name = "c_type", length = 50, nullable = false, comment = "灾害类型")
	@JsonProperty("type")
	private String type;
	
	@Column(order = 50, name = "c_level", length = 10, nullable = false, comment = "灾害等级")
	@JsonProperty("level")
	private String level;
	
	@Column(order = 60, name = "c_status", length = 10, nullable = true, comment = "灾害状态")
	@JsonProperty("status")
	private String status;
	
	@Column(order = 70, name = "c_description", length = 1000, nullable = true, comment = "灾害描述")
	@JsonProperty("description")
	private String description;

	public String getCityId() {
		return cityId;
	}

	public void setCityId(String cityId) {
		this.cityId = cityId;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getPath() {
		return path;
	}

	public void setPath(String path) {
		this.path = path;
	}

	public Date getPubTime() {
		return pubTime;
	}

	public void setPubTime(Date pubTime) {
		this.pubTime = pubTime;
	}

	public Date getInsertTime() {
		return insertTime;
	}

	public void setInsertTime(Date insertTime) {
		this.insertTime = insertTime;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getLevel() {
		return level;
	}

	public void setLevel(String level) {
		this.level = level;
	}

	public String getStatus() {
		return status;
	}

	public void setStatus(String status) {
		this.status = status;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}
}
